/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*Structure and parameter definitions for Xdl xview-object handling routines*/

#ifndef __XDL_VIEW_H__
#define __XDL_VIEW_H__
#include <stdio.h>
#include <stdlib.h>

#include "xdl_systyp.h"

#ifdef VMS
#include <file.h>
#include <iodef.h>
#include <decw$include/Xlib.h>
#include <decw$include/Intrinsic.h>
#include <decw$include/Xutil.h>
#include <decw$include/Xos.h>
#include <decw$include/cursorfont.h>
#include <decw$include/keysym.h>
#include <decw$include/Xatom.h>
#include <decw$include/Xresource.h>
#else
#ifdef HAVE_FNCTL_H
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include <signal.h>

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#endif

#define MAXCOLORSETS 20      /* Maximum number of color sets allowed */

#define BORDER_WIDTH 4       /* Minimum left/right border width for normal use 
                                - also used as offset for top left of active 
                                strip area */
#define BORDER_HEIGHT 4      /* Minimum top/bottom border height for normal
                                use - also used as offset for top left of 
                                active strip area */
#define ACT_STRIP_H 20       /* height of the active strip area */
#define ACT_STRIP_HEIGHT 24  /* Total height required for active strip from
                                top of area to bottom of the strip =
                                ACT_STRIP_H + BORDER_HEIGHT */
#define MAXPOINTERS 20       /* Maximum no. of pointers for XDLSTR transfers */

#define FBLANK ' '

typedef struct
{
   char *gd;              /*Pointer to a private data area used by the 
                            library window function*/
   void (*callback)();    /*Callback routine to return data to view-object*/
   void (*tidy_up)();     /*Tidy up routine to be called when library routine
                            window is deleted via xdl_delete_view_object (this 
                            need not free gd as this will be done automatically 
                            by xdl_delete_view_object)*/
}
   XDl_libdata;

typedef struct 
{
   Window wid;            /*Window i.d. for the window*/
   void (*repaint)();     /*Repaint routine to be called on expose,
                            configure notify (resize) */
   void (*ehp)();         /*Event Handling Procedure for other events*/
   XDl_libdata *libdata;  /*Pointer to XDl_libdata structure for a library
                            routine window - otherwise NULL*/
}
   XDl_window;

typedef struct
{
   int        vh;         /*View object handle*/
   Window     wid;        /*The (top level) window id for this view-object*/
   int        vh_parent;  /*View object handle of parent if view-object is
                            a sub_window of another view-object in the
                            XDL_view_objects list*/
   int        type;       /*A unique integer value identifying the
                            view-object type. This is for safety checking
                            to ensure that, when a routine is called to
                            manipulate or get data from a view-object
                            identified by the view-object handle, the
                            view-object is of the correct type*/
   char *gd;              /*Pointer to structure of data items used for
                            communicating between the functions of the
                            view-object*/
   int        num_win ;   /*Number of window objects associated with
                            this view-object (subwindows etc.)*/
   int     on_off_state;  /*Current program awaiting input on/off state flag
                            =0 not waiting; =1 waiting for input*/
   XDl_window *win_list;  /*Array of  window objects associated with 
                            this view-object. (includes the top level
                            window of the view-object*/ 
   void    (*on_off)();   /*Pointer to view-object's on/off function to
                            be called to turn program input from the view-
                            object on or off. This will be called by
                            the xdl_get_events routine. A null may
                            be given if such a routine is not required
                            or implemented*/
   void (*tidy_up)();     /*Pointer to the view_object's tidy up function to
                            be called when the view-object is deleted by 
                            calling the xdl_delete_view_object routine. This
                            routine will for example free any memory areas
                            allocated by the view object. (It need not free
                            the global data area as this will be automatically
                            freed by the xdl_delete_view_object routine). A
                            NULL may be given if such a routine is not required
                            */
}
   XDl_view_object;

typedef struct
{
   Colormap      colormap;       /*The color map from which the color cells
                                   were allocated*/
   unsigned int  ncolors;        /*The number of colors allocated*/
   unsigned long *pixels;        /*Array of the pixel values rturned from
                                   the allocation of the color cells*/
   unsigned int  nplanes;        /*No. of overlay planes*/
   unsigned long *plane_masks;   /*Array of plane masks returned from the
                                   allocation of the color cells*/
}
   XDl_colorset;
   
/* Function Declarations */
/* ===================== */

/* xdl_view.c */

int xdl_open_view (int numc, int * ncolors, int * nplanes);
int xdl_get_events (int num_vh, int * vh_list);
void xdl_open_file (char * filnam, int lenf, int *fid);
void xdl_set_file_input (int fid);
void xdl_clear_file_input ();
int xdl_file_events (int num_vh, int * vh_list);
int xdl_get_record (char * string, int max_len);
void xdl_flush_events (int dummy);
int xdl_delete_view_object (int vh);
int xdl_view_object_rootxy (int vh, int x, int y, int * xroot, int * yroot);
int xdl_view_object_raise (int vh);
int xdl_view_object_lower (int vh);
int xdl_get_events_timeout (int num_vh, int * vh_list, int msecs, 
                            int num_vh2, int * vh_list2);
int xdl_getio_events (int num_vh, int * vh_list);
int xdl_getio_string (char * string, int max_len);
int xdl_tread (char * string, int max_len);
int xdl_timer_events (int on_off);
void xdl_sync (int opt);
void xdl_delay (int msec);
Window xdl_create_base_frame_window (char * title, char * icon_label, 
                                     int x, int y, 
                                     int width, int height);
Window xdl_create_frame_window (Window wparent, int x, int y, 
                                int width, int height, int cset);
Window xdl_create_paint_window (Window wparent, int x, int y, int brdw, 
                                int width, int height, int cset);
Window xdl_create_active_strip (Window wparent, int vh, int width, 
                                void (*repaint)());
void xdl_active_strip_on (Window wid, char * str, int bold);
void xdl_active_strip_off (Window wid);
int xdl_new_view_object (int vh, Window wid, int vh_parent, char * gd, 
                         int type, void (*on_off)(), void (*tidy_up)());
void xdl_add_window (int vh, Window wid, void (*repaint)(), void (*ehp)());
void xdl_add_libwin (int vh, Window wid, char * gd, void (*repaint)(), 
                     void (*ehp)(), void (*callback)(), void (*tidy_up)());
void xdl_add_top_callbacks (int vh, void (*repaint)(), void (*ehp)());
int xdl_getiv (int vh);
char * xdl_get_global (Window wid);
int xdl_get_vh (Window wid);
void xdl_delete_panel_item (int vh, Window wid);
void xdl_select_owner (Window wid);
void xdl_select_cancel ();
void xdl_select_respond (XEvent * event, char * str, int nitems);
void xdl_select_request (XEvent * event, Window wid);
int xdl_select_receive (XEvent * event, Window wid, char **str,
                        int * nitems, int maxinstring);
void xdl_view_errmsg (char * routine_name, char * message);
void xdl_view_iv_errmsg (char * routine_name, int vh);
void xdl_view_type_errmsg (char * routine_name, int vh, 
                           char * view_object_type);
void xdl_rootxy (Window wid, int x, int y, int * xroot, int * yroot);
void xdl_copy_chars (char * string_in, char * string_out, int max_len);
int xdl_blank_string (char * str);
void xdl_chop_blanks (char * str);
int xdl_ndigit (int i);
void xdl_upc (char * str);
void xdl_lwc (char * str);
void xdl_upcc (char * ch);
void xdl_lwcc (char * ch);
int xdl_rand16 ();
void xdl_srand16 (unsigned int seed);
void xdl_ch_size (int font_type, int opt, int * ch_width, int * ch_height, 
                  int * ascent, int * descent);
int xdl_set64gray (int cset, int contrast_val, int type);
int xdl_set64gray_yel (int cset, int contrast_val, int type);
int xdl_set64color (int cset, int contrast_val, int type);
int xdl_get64gray (unsigned long * pixels, int contrast_val, int type);
int xdl_get64gray_yel (unsigned long * pixels, int contrast_val, int type);
int xdl_get64color (unsigned long * pixels, int contrast_val, int type);
void xdl_delete_children (int vh_parent);
int xdl_any_children (int vh);

#ifdef XDL_BUILDING
void xdl_free (void *);
#define free xdl_free
#endif

/* xdl_base_fr.c */

void xdl_base_frame (int vh, int width, int height, char * title, 
                     int title_len, char * icon_label, int icon_len, 
                     int x, int y);

/* xdl_menu_area.c */

int xdl_menu_area (int vh, int vh_parent, int x, int y, int cset, 
                   int max_items, int max_name_chars, int font_type, 
                   int quit_box, int max_title_len, 
                   int min_width, int min_height);
int xdl_menu_area_setmenu (int vh, int num_items, char * names, 
                           int name_len, char * title, int title_len, 
                           char * quit_name, int quit_len, int font_type);
int xdl_menu_area_getitem (int vh, int * item, int * quit);
int xdl_menu_area_getrootxy (int vh, int * xroot, int * yroot);
void xdl_menu_area_getsize (int max_items, int max_name_chars, 
                            int font_type, int quit_box, 
                            int max_title_len, int * w, int * h);

/* xdl_parm_tab.c */

int xdl_param_table (int vh, int vh_parent, int x, int y, int cset, 
                     int num_cols, int num_rows, int max_name_chars, 
                     int max_value_disp, int max_value_chars, 
                     char * title, int nt, int font_type,
                     int menus_allowed);
int xdl_param_table_getvalue (int vh, int * item, char * string, 
                              int max_len);
int xdl_param_table_error (int vh, int item, char * errstr, int len);
int xdl_param_table_setitem (int vh, int item, char * name, int len_name, 
                             char * value, int len_value, int toggle, 
                             int emphasize);
int xdl_param_table_setvalue (int vh, int item, char * value, 
                              int len_value, int toggle);
int xdl_param_table_setmenu (int vh, int item, int num_menu_vals, 
                             char * values, int len_vals);
int xdl_param_table_setbutton (int vh, int item, char * values, 
                               int len_vals);
int xdl_param_table_clrmenu (int vh, int item);
int xdl_param_table_silent (int vh, int num_items, int * items);
int xdl_param_table_normal (int vh, int num_items, int * items);
int xdl_param_table_bold (int vh, int num_items, int * items);
int xdl_param_table_all_normal (int vh);
int xdl_param_table_delitem (int vh, int item);
int xdl_param_table_clearitems (int vh);
int xdl_param_table_newtitle (int vh, char * title, int nt);
void xdl_param_table_getsize (int num_cols, int num_rows, 
                              int max_name_chars,
                              int max_value_disp, int nt, int font_type, 
                              int menus_allowed, int * w, int * h);

/* xdl_io_window.c */

int xdl_io_window (int vh, int vh_parent, int x, int y, int cset, 
                   int ncols, int nrows, int min_width, 
                   int min_height, int font_type, int nscroll);
int xdl_io_window_print (int vh, char * string, int len_str);
int xdl_io_window_getstring (int vh, char * string, int max_len);
int xdl_io_window_input_message (int vh, char * message, int len);
int xdl_io_window_set_focus (int vh, int msec, int ntry);
int xdl_io_window_rootxy (int vh, int * xroot, int * yroot);
void xdl_io_window_getsize (int ncols, int nrows, int font_type, 
                            int nscroll, int * w, int * h);

/* xdl_text_tab.c */

int xdl_text_table (int vh, int vh_parent, int x, int y, int cset, 
                    int ncols, int nrows, int min_width, 
                    int min_height, int font_type);
int xdl_text_table_cell (int vh, int vh_parent, int x, int y, int cset, 
                         int iact, int max_cells, int ncols, int nrows, 
                         int min_width, int min_height, int font_type,
                         void (*highl_fn)());
int xdl_text_table_text (int vh, char * string, int len_str, int row, 
                         int col, int bold);
int xdl_text_table_symbol (int vh, int symb, int row, int col, int colr);
int xdl_text_table_clear (int vh);
int xdl_text_table_clearsect (int vh, int row1, int col1, 
                              int row2, int col2);
int xdl_text_table_input_message (int vh, char * message, int len);
int xdl_text_table_define_cell (int vh, int num, int row1, int col1, 
                                int row2, int col2, int hightyp,
                                int high_nosel, int colr_bg, int colr_l);
int xdl_text_table_clear_cell (int vh, int num);
int xdl_text_table_getinp (int vh, int * num);
void xdl_text_table_getsize (int ncols, int nrows, int font_type, 
                             int * w, int * h);
void xdl_text_table_getcsize (int ncols, int nrows, int font_type, 
                              int iact, int * w, int * h);

/* xdl_graph_win.c */

int xdl_graph_win (int vh, int vh_parent, int x, int y, int cset, 
                   int ncolrs, int gr_width, int gr_height, 
                   int min_width, int min_height, 
                   int expand, int posn);
int xdl_graph_win_map (int vh, float ax1_min, float ax1_max, 
                       float ax2_min, float ax2_max, 
                       float dax1_min, float dax1_max, 
                       float dax2_min, float dax2_max, int ityp);
int xdl_graph_win_style (int vh, int fg_colr, int line_width, 
                         int font_type);
int xdl_graph_win_set_colour (int vh, int icolr, int red, int blue, 
                              int green);
int xdl_graph_win_line (int vh, float ax1_start, float ax2_start, 
                        float ax1_end, float ax2_end);
int xdl_graph_win_polyline (int vh, float * ax1_coords, float * ax2_coords,
                            int  npoints);
int xdl_graph_win_stext (int vh, float ax1_coord, float ax2_coord, 
                         char * txt, int lentxt, float size1,
                         float size2, int sizopt, int cent, 
                         float angle);
int xdl_graph_win_xtext (int vh, float ax1_coord, float ax2_coord, 
                         char * txt, int lentxt, int cent);
int xdl_graph_win_symbol (int vh, float ax1_coord, float ax2_coord, 
                          int symb, float size, int sizopt);
int xdl_graph_win_polysymbol (int vh, float * ax1_coords, 
                              float * ax2_coords, int npoints, int symb,
                              float size, int sizopt);
int xdl_graph_win_rectangle (int vh, float ax1_start, float ax2_start, 
                             float ax1_end, float ax2_end, int fill);
int xdl_graph_win_polygon (int vh, float * ax1_coords, float * ax2_coords,
                           int npoints, int fill);
int xdl_graph_win_arc (int vh, float ax1_cen, float ax2_cen, 
                       float rad1, float rad2, float angle_start,
                       float angle_range, int fill);
int xdl_graph_win_ax1 (int vh, float ax1_orig, float ax2_orig, float axlen,
                       int  ndiv, float ax1_div, int nd,
                       float htick, float siz, int sizopt);
int xdl_graph_win_ax2 (int vh, float ax1_orig, float ax2_orig, float axlen,
                       int ndiv, float ax2_div, int nd,
                       float wtick, float siz, int sizopt);
int xdl_graph_win_getcurs (int vh, float * ax1_coord, float * ax2_coord);
int xdl_graph_win_input_message (int vh, char * message, int len);
int xdl_graph_win_setid (int vh, int id);
int xdl_graph_win_delete (int vh, int id);
void xdl_graph_win_getsize (int gr_width, int gr_height, int * w, int * h);

/* xdl_pop_notic.c */

int xdl_popup_notice (int xroot, int yroot, char * msg1, int lenmsg1, 
                      char * msg2, int lenmsg2, 
                      char * butstr1, int lenbutstr1, 
                      char * butstr2, int lenbutstr2, 
                      int font_type, int arrow);


/* xdl_pop_dialg.c */

int xdl_popup_dialog (int xroot, int yroot, char * prompt, int lenp, 
                      int width, char * reply, int max_len, 
                      int font_type, int bold, int arrow);

/* xdl_pop_menu.c */

int xdl_popup_menu (int xroot, int yroot, int num_items, char * strings, 
                    int len_strs, int font_type, int brdr_colr, int cent);

/* xdl_pop_frame.c */

void xdl_popup_frame (int vh, int xroot, int yroot, 
                      int width, int height, int brdr_colr);

/* xdl_prog_bar.c */

void xdl_progress_bar (int vh, int xroot, int yroot, char * title, 
                       int title_len, int ind_len, 
                       int max_val, int font_type, int colr);
int xdl_progress_bar_value (int vh, int value);


/* xdl_control.c */

int xdl_control_choice (int vh, int vh_parent, int x, int y, 
                        int act_strip, int hcen, int vcen,
                        int font_type, int brdr, char * label, int llen, 
                        int num_items, char * strings, int len_strs, 
                        int df_item, char * msg, int len_msg, 
                        int minw, int minh);
int xdl_control_choice_reset (int vh, int item);
int xdl_control_choice_getdata (int vh, int * item);
void xdl_control_choice_getsize (int act_strip, int font_type, int brdr, 
                                 int llen, int len_strs, 
                                 int * width, int * height);
int xdl_control_button (int vh, int vh_parent, int x, int y, 
                        int act_strip, int hcen, int vcen,
                        int font_type, int brdr, char * label, 
                        int llen, int maxlen,
                        char * msg, int len_msg,
                        int bold, int cent, int minw, int minh);
int xdl_control_button_set (int vh, char * label, int llen, 
                            int bold, int cent);
void xdl_control_button_getsize (int act_strip, int font_type, int brdr, 
                                 int maxlen, int * width, int * height);
int xdl_control_label (int vh, int vh_parent, int x, int y, 
                       int hcen, int vcen, int font_type, int brdr, 
                       char * label, int llen, int maxlen,
                       int bold, int minw, int minh);
int xdl_control_label_set (int vh, char * label, int llen, int bold);
void xdl_control_label_getsize (int font_type, int brdr, int maxlen,
                                int * width, int * height);
int xdl_control_value (int vh, int vh_parent, int x, int y, 
                           int act_strip, int hcen, int vcen,
                           int font_type, int brdr, 
                           char * label, int llen, 
                           char * value, int vlen, int maxvlen,
                           char * msg, int len_msg,
                           int minw, int minh);
int xdl_control_value_set (int vh, char * value, int vlen);
int xdl_control_value_getdata (int vh, char * valstr, int maxv);
void xdl_control_value_getsize (int act_strip, int font_type,int  brdr, 
                                int llen, int maxvlen,
                                int * width, int * height);
int xdl_control_slider (int vh, int vh_parent, int x, int y, 
                        int act_strip, int hcen, int vcen,
                        int font_type, int brdr, char * label, int llen, 
                        int travel_len, int startpos,
                        char * msg, int len_msg,
                        int minw, int minh);
int xdl_control_slider_reset (int vh, int islide);
int xdl_control_slider_getdata (int vh, int * ipos, int * travel_len);
void xdl_control_slider_getsize (int act_strip, int font_type, int brdr, 
                                 int llen, int travel_len, 
                                 int * width, int * height);
int xdl_control_io (int vh, int vh_parent, int x, int y, int act_strip, 
                    int hcen, int vcen, int font_type, int brdr, 
                    int max_pr, int min_rp, int max_rp, int ch_count,
                    char * msg, int len_msg, int minw, int minh);
int xdl_control_io_prompt (int vh, char * prompt, int plen, int iflag);
int xdl_control_io_clear (int vh);
int xdl_control_io_kbfocus (int vh);
int xdl_control_io_getdata (int vh, char * reply, int maxr, 
                            int * iret, int * iflag);
void xdl_control_io_getsize (int act_strip, int font_type, int brdr, 
                             int max_pr, int min_rp,
                             int max_rp, int ch_count, 
                             int * width, int * height);

/* xdl_blank_obj.c */

int xdl_blank_object (int vh, int vh_parent, int x, int y, int cset, 
                      int width, int height);

/* xdl_image.c */

int xdl_image (int vh, int vh_parent, int x, int y, int cset, 
               char * imgdata, int type, int ns_ras, int nf_ras, int nf_off,
               int o_ax1, int o_ax2,int  nreq_ax1, int nreq_ax2, 
               int ncmp_in, int iorder, int jorder,
               int min_val, int max_val, int min_thresh, int max_thresh, 
               int  min_width, int min_height, int bg_typ, int ovly_typ,
               int contrast, int disp_now);
int xdl_image_getpix (int vh, int * ax1pix_img, int * ax2pix_img);
int xdl_image_getpix_ovly (int vh, int ov_num, int * ax1pix_img, 
                           int * ax2pix_img);
int xdl_image_getrect (int vh, int * ax1pix_dn, int * ax2pix_dn, 
                       int * ax1pix_up, int * ax2pix_up,
                       int vect_id, int color, int ov_num);
int xdl_image_settings (int vh, int * min_thr, int * max_thr, int * mono_thr,
                        int * contrast, int * color_opt, int * ovly_opt, 
                        int * bg_opt, int * mag_opt, int * show_int);
int xdl_image_reset (int vh, int min_thr, int max_thr, int mono_thr, 
                     int contrast, int color_opt, int ovly_opt, int bg_opt, 
                     int mag_opt, int show_int);
int xdl_image_axnames (int vh, char * name1, int len1, char * name2, 
                       int len2);
int xdl_image_display_now (int vh);
int xdl_image_background (int vh, char * bg_data, int bg_cmp, int nf_bgoff);
int xdl_image_newimg (int vh, char * imgdata, int disp_now);
int xdl_image_ovly_offset (int vh, int xoff, int yoff);
int xdl_image_set_colour (int vh, int icolr, int red, int blue, int green);
int xdl_image_set_colormap (int vh, int * red, int * green, int * blue);
int xdl_image_symbol (int vh, int ax1_pixel, int ax2_pixel, int symbol, 
                      int color, int ov_num);
int xdl_image_symbols (int vh, int nsymb, int * ax1_pixel, int * ax2_pixel, 
                       int * symbol, int * color, int * ov_num, int nt);
int xdl_image_clear_symbols (int vh);
int xdl_image_del_symbol (int vh, int ax1_pixel, int ax2_pixel, 
                          int symbol, int color, int ov_num);
int xdl_image_vect (int vh, int vect_id, int ax1_1_pixel, int ax2_1_pixel, 
                    int ax1_2_pixel, int ax2_2_pixel, int color, 
                    int ov_num, int magw_upd);
int xdl_image_vects (int vh, int vect_id, int num_vect, 
                     int * ax1_1_pixel, int * ax2_1_pixel, 
                     int * ax1_2_pixel, int * ax2_2_pixel, 
                     int color, int ov_num, int magw_upd);
int xdl_image_vects_pos (int vh, int vect_id, int num_vect, 
                         int * ax1_1_pixel, int * ax2_1_pixel, 
                         int * ax1_2_pixel, int * ax2_2_pixel, 
                         int * ax1_1_pos, int * ax2_1_pos,
                         int * ax1_2_pos, int * ax2_2_pos, 
                         int color, int ov_num, int magw_upd);
int xdl_image_boxes (int vh, int vect_id, int num_box, 
                     int * ax1_1_pixel, int * ax2_1_pixel, 
                     int * ax1_2_pixel, int * ax2_2_pixel, 
                     int ipos, int color, int ov_num, int magw_upd);
int xdl_image_clear_vects (int vh);
int xdl_image_del_vect (int vh, int vect_id);
int xdl_image_text (int vh,int  text_id, int ax1_pixel, int ax2_pixel, 
                    char * string, int len, int color, int font_type, 
                    int bold, int ov_num);
int xdl_image_del_text (int vh, int text_id);
int xdl_image_clear_text (int vh);
int xdl_image_input_message (int vh, char * message, int len);
void xdl_image_getsize (int nreq_ax1, int nreq_ax2, int iorder, 
                        int jorder, int ncmp, int * w, int * h);

/* xdl_laue_sim.c */

int xdl_laue_sim (int vh, int vh_parent, int x, int y, int cset, int type, 
                  int size, int nspot, int * hkl, float * xf, float * yf, 
                  float * lam, float * dthr, int * multi, int * flags, 
                  int * nodhkls, int * idxlam, int * idxdmin, int * idxnod, 
                  float filmr, float gnomr, float lmin, float lmax,  
                  float dmin, float d_lmin, float d_lmax, float d_dmin, 
                  float * cell, float * phis, int * latt, float ctof,
                  int min_width, int min_height);
void xdl_laue_sim_getsize (int type, int size, int * w, int * h);
int xdl_laue_sim_new (int vh, int nspot, int * hkl, float * xf, float * yf, 
                      float * lam, float * dthr, int * multi, int * flags, 
                      int * nodhkls, int * idxlam, int * idxdmin, 
                      int * idxnod, float filmr, float gnomr,
                      float lmin, float lmax, float dmin, float * cell, 
                      float * phis, int * latt, float ctof);

/* xdl_rot_sim.c */

int xdl_rot_sim (int vh, int vh_parent, int x, int y, int cset, int type, 
                 int size, int iord, int mindx, int icrys, int imgnum, 
		 float filmr, float * scax, float rots, float rote, 
                 int isyn, float eta, float divv, float divh, float delamb, 
                 float delcor, int iax_v, int iax_h, int nwmax, 
                 float cur_rotr, float cur_eta, float cur_divv, 
                 float cur_divh, float cur_delamb, 
                 float * cell, float * umat, float * delphi, 
                 float wave, int * lsys, float ctod,
                 char * name1, int len1, char * name2, int len2, 
                 int min_width, int min_height);
void xdl_rot_sim_getsize (int type, int size, int * w, int * h);
int xdl_rot_sim_new (int vh, int iord, char * name1, int len1, 
                     char * name2, int len2, int mindx, int icrys, 
                     int imgnum, float filmr, float * scax, 
                     float rots, float rote, int isyn, float eta, 
                     float divv, float divh, float delamb, float delcor, 
                     int iax_v, int iax_h, int nwmax, float * cell, 
                     float * umat, float *delphi, float wave, int * lsys, 
                     float ctod); 

/* xdl_show_unq.c */

int xdl_show_unq (int vh, int vh_parent, int x, int y, int cset, int size, 
                  int mindx_unq, float dstmax, float bmatr[3][3], 
                  char * title, int lent, int min_width, int min_height);
int xdl_show_unq_getdata (int vh, float * x, float * y);

/* xdl_img_rd.c */

int xdl_open_film_file (char * filename, int lenf, int * fd);
int xdl_read_full_film (int fd, int order, int nxrasts, int nyrasts, 
                        int vh_bar, int xroot, int yroot, 
                        unsigned char * img_full);
int xdl_close_film_file (int fd);
int xdl_open_i2 (char * filename, int lenf, int * fd);
int xdl_read_full_i2 (int fd, int order, int nxrasts, int nyrasts, 
                      int vh_bar, int xroot, int yroot, 
                      unsigned short * img_full, 
                      int * minval, int * maxval);
int xdl_close_i2 (int fd);
int xdl_axord (char * str, int len, int * order, int * swaptyp);
int xdl_read_mar (int fd, int vh_bar, int xroot, int yroot, 
                  int * img_full, int * nxpix, int * nypix, 
                  int * ns_ras, int * nf_ras, int * minval, int * maxval);
int xdl_rdsquash_mar (int fd, int vh_bar, int xroot, int yroot, 
                      unsigned short * img_full, int * nxpix, int * nypix, 
                      int * ns_ras, int * nf_ras,int * minval, int * maxval);
int xdl_read_pfbyte (int fd, int order, int nxrasts, int nyrasts, 
                     int vh_bar, int xroot, int yroot, 
                     unsigned short * img_full);

/* xdl_bg.c */

int xdl_bg_calc (char * img, int type, int ns_rasts, int nf_rasts, 
                 int nf_off, int ncmp, int npbox, 
                 int percent, char * img_bg, int * nf_bgoff);
int xdl_bg_calc_prog (char * img, int type, int ns_rasts, int nf_rasts, 
                      int nf_off, int ncmp, int npbox, 
                      int percent, char * img_bg, int * nf_bgoff, 
                      void (*progress)(), void (*cancel)());
int xdl_bg_strip (char * img, int type, int ns_rasts, int nf_rasts, 
                  int nf_off, int ncmp, int npbox, 
                  int percent, int iopt, int * bgave, int * maxr_off);
int xdl_bg_radimg (int type, int ns_bg, int nf_bg, int nf_bgoff, 
                   int * bgave, int maxr_off, char * img_bg);
int xdl_bg_val (int islow, int ifast,int *  bgave, int maxr_off, 
                int ns_rasts, int nf_rasts, int ncmp, int * ival);
int xdl_bg_valimg (int islow, int ifast, char * img_bg, int type, 
                   int nf_bgoff, int ncmp, int * ival);

/* xdl_layout.c */

int xdl_layout_init ();
int xdl_layout_set_single(char * name, int len, int w, int h, 
                          int hjust, int vjust);
int xdl_layout_set_table (char * name, int len, int nrows, int ncols, 
                          char * names, int name_len, int order, 
                          int w, int h);
int xdl_layout_set_pair (int h_or_v, int just, char * name, int len, 
                         char * name_child1, int len1,
                         char * name_child2, int len2, 
                         int w, int h, int sep, int percent);
int xdl_layout (char * name, int len, int x, int y, int * w, int * h);
int xdl_layout_get_position (char * name, int len, int * x, int * y, 
                             int * w, int * h);
int xdl_layout_reset_table (char * name, int len, int x, int y, 
                            int w, int h);
int xdl_layout_reset_single (char * name, int len, int x, int y, 
                             int w, int h);
#endif /* ! __XDL_VIEW_H__ */
